
#include <vdr/remote.h>
#include <vdr/keys.h>

#include "key_learner.h"

// --- cMenuEditKey -----------------------------------------------------

#define LEARN_TIMEOUT_MS  2000 // timeout for learning keys in ms

cMenuEditKey::cMenuEditKey(const char* name, cRemote *_remote, cKey *_key, bool New)
:cMenuEditItem(name)
{
   learning_state = LEARN_OFF;
   remote = _remote;
   if (New) {
      key = new cKey(_key->Remote(), "NO_CODE", _key->Key());
      Keys.Add(key, _key);
   } else {
      key = _key;
   }
   Set();
}

cMenuEditKey::~cMenuEditKey()
{
   cRemote::SetLearning(NULL);
}

void cMenuEditKey::Set(void)
{
   switch (learning_state) {
      case LEARN_GET_KEY:
         SetValue("Learning this key");
         break;
      case LEARN_TIMEOUT:
         //SetStatus("TIMEOUT")
         SetValue("TIMEOUT");
         break;
      case LEARN_ABORTED:
         SetValue("ABORTED");
         break;
      default: {
         const char *tmp=key ? key->Code() : 0;
         if (!tmp) {
            SetValue("NO CODE");
	    break;
	 }

         bool try_to_decode_key=true;

	 // Guess if this is a number encoded to a hex string
         if (try_to_decode_key && strlen(tmp)==16 && tmp[0]=='0') {

	    // skip leading zeros
            while (tmp[0] == '0' && tmp[1] != '\0')
               tmp++;

	    // decode to integer
            long long code = strtoll(tmp, NULL, 16);
            char out[60];
            if (code==0x20)
	       SetValue(tr("Keycode$space"));
            else if (code>0x20 && code<0x7f) {
	       snprintf(out, sizeof(out), "%c", char(code));
	       SetValue(out);
	    } else {
	       snprintf(out, sizeof(out), "0x%s", tmp);
	       SetValue(out);
	    }
         } else {
	    // Normal value
            SetValue(tmp);
	 }
      }
   }
}

// Query key
void cMenuEditKey::Action()
{
   char *NewCode=NULL;
   eKeys Key = cRemote::Get(LEARN_TIMEOUT_MS, &NewCode);

   // Code of key is same as already set
   if (Key == key->Key()) {
      learning_state = LEARN_ABORTED;
      FinishLearning();
      return;
   }

   switch (Key) {
      case kNone:
         if (NewCode)
            SetLearnResult(NewCode);
         else
            learning_state = LEARN_TIMEOUT;
         break;
      default:
         learning_state = LEARN_ABORTED;
	 break;
   }
   FinishLearning();
}

void cMenuEditKey::SetLearnResult(char * NewCode)
{
   //esyslog("NewCode=%s", NewCode);

   cKey *newkey = new cKey(key->Remote(), NewCode, key->Key());
   Keys.Ins(newkey, key);
   Keys.Del(key, true);

   key = newkey;
   Keys.Save();
}

void cMenuEditKey::StartLearning()
{
   learning_state = LEARN_GET_KEY;
   Set();
   cRemote::SetLearning(remote);
}

void cMenuEditKey::FinishLearning()
{
   if (learning_state == LEARN_GET_KEY)
      learning_state = LEARN_DONE;
   cRemote::SetLearning(NULL);
   Set();
}

eOSState cMenuEditKey::ProcessKey(eKeys Key)
{
   if (Key == kNone) {
      if (learning_state == LEARN_GET_KEY) {
         Action();
      }
      return osContinue;
   }
   // not learning
   switch (Key) {
      case kOk:
         StartLearning();
         break;
      default: return cMenuEditItem::ProcessKey(Key);;
   }
   return osContinue;
/*   if (autoAdvanceTimeout.TimedOut()) {
      SetLearning(false);
   }*/
}


