/*
 * keyconf.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>
#include <vdr/remote.h>
#include <vdr/keys.h>

#include "i18n.h"
#include "menu_remote_list.h"

static const char *VERSION        = "0.0.1";
static const char *DESCRIPTION    = "Allows online reconfiguration of keys";
static const char *MAINMENUENTRY  = "Keyconf";

class cPluginKeyconf : public cPlugin {
private:
  // Add any member variables or functions you may need here.
  bool do_key_check;
  bool bForceLearning;
  void CheckKeys(void);
public:
  cPluginKeyconf(void);
  virtual ~cPluginKeyconf();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Stop(void);
  virtual void Housekeeping(void);
  virtual void MainThreadHook(void);
  virtual cString Active(void);
  virtual time_t WakeupTime(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  virtual bool Service(const char *Id, void *Data = NULL);
  virtual const char **SVDRPHelpPages(void);
  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
  };

cPluginKeyconf::cPluginKeyconf(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
  do_key_check = true;
  bForceLearning = false;
}

cPluginKeyconf::~cPluginKeyconf()
{
  // Clean up after yourself!
}

const char *cPluginKeyconf::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginKeyconf::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginKeyconf::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  RegisterI18n(Phrases);  
  return true;
}

bool cPluginKeyconf::Start(void)
{
  // Start any background activities the plugin shall perform.
  return true;
}

void cPluginKeyconf::Stop(void)
{
  // Stop any background activities the plugin shall perform.
}

void cPluginKeyconf::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

void cPluginKeyconf::MainThreadHook(void)
{
  // Perform actions in the context of the main program thread.
  // WARNING: Use with great care - see PLUGINS.html!
  if (do_key_check) {
     do_key_check = false;
     CheckKeys();
  }
}

void cPluginKeyconf::CheckKeys(void)
{
   bool need_reconf=false;
   if (Keys.Count() == 0) {
      // No single key defined - reconf
      bForceLearning = true;
      printf("Forced learning\n");
      cRemote::CallPlugin("keyconf");
   }
}

cString cPluginKeyconf::Active(void)
{
  // Return a message string if shutdown should be postponed
  return NULL;
}

time_t cPluginKeyconf::WakeupTime(void)
{
  // Return custom wakeup time for shutdown script
  return 0;
}

cOsdObject *cPluginKeyconf::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  cRemoteListMenu* pMenu = NULL;

  pMenu = new cRemoteListMenu();
  if (!pMenu)
     return NULL;

  if (bForceLearning)
     pMenu->ForceLearning();
  return pMenu;
}

cMenuSetupPage *cPluginKeyconf::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return NULL;
}

bool cPluginKeyconf::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  return false;
}

bool cPluginKeyconf::Service(const char *Id, void *Data)
{
  // Handle custom service requests from other plugins
  return false;
}

const char **cPluginKeyconf::SVDRPHelpPages(void)
{
  // Return help text for SVDRP commands this plugin implements
  return NULL;
}

cString cPluginKeyconf::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
{
  // Process SVDRP commands this plugin implements
  return NULL;
}

VDRPLUGINCREATOR(cPluginKeyconf); // Don't touch this!
