
#include <vdr/remote.h>
#include <vdr/keys.h>
#include <vdr/interface.h>

#include "menu_remote_view.h"
#include "key_learner.h"

// --- cRemoteViewMenu -----------------------------------------------------

cRemoteViewMenu::cRemoteViewMenu(cRemote *_remote)
:cOsdMenu("",20)
{
   char *Buffer = NULL;
   remote = _remote;

   // Menu name
   asprintf(&Buffer, "View Remote (%s)", remote->Name());
   SetTitle(Buffer);
   free(Buffer);

   /*
   // Details
   asprintf(&Buffer, "Name:\t%s", remote->Name());
   Add(new cOsdItem(Buffer));
   free(Buffer);
   */

   cKey *key=Keys.First();
   while (key) {
      if (strcmp(key->Remote(), remote->Name()) == 0)
         Add(new cMenuEditKey(cKey::ToString(key->Key()), remote, key));
      key = Keys.Next(key);
   }

/*   asprintf(&Buffer, "Last:\t%s",
      (const char*)TimeString(remote->LastActivity()));
   Add(new cOsdItem(Buffer));
   free(Buffer); */
   //cRemote::SetLearning(remote);
   SetHelp(tr("Button$Edit"), tr("Button$New"), tr("Button$Delete"), tr("Button$Duplicate"));
}

cRemoteViewMenu::~cRemoteViewMenu()
{
   cRemote::SetLearning(NULL);
}

eOSState cRemoteViewMenu::ProcessKey(eKeys Key)
{
   eOSState state = cOsdMenu::ProcessKey(Key);
   cMenuEditKey *editor;
   if (state == osUnknown) {
      switch (Key) {
         case kRed:
	    editor = CurrentKeyEditor();
            if (editor)
               editor->StartLearning();
            state = osContinue;
	    break;
         case kYellow:
	    state = Delete();
	    break;
         case kGreen:
	    state = New();
	    break;
         case kBlue:
	    state = Duplicate();
	    break;
	 default:
	    break;
      }
   }
   return state;
}

cMenuEditKey* cRemoteViewMenu::CurrentKeyEditor(void)
{
   cMenuEditKey *item = dynamic_cast<cMenuEditKey*>( Get(Current()) );
   if (item)
      return item;

   return NULL;
}

cKey* cRemoteViewMenu::CurrentKey(void)
{
   cMenuEditKey *item = CurrentKeyEditor();
   if (item)
      return item->Key();

   return NULL;
}

eOSState cRemoteViewMenu::Delete()
{
   cKey *curKey = CurrentKey();
   if (curKey) {
      if (Interface->Confirm(tr("Edit$Delete Key?"))) {
          Keys.Del(curKey, true);
          //Keys.Save();
          cOsdMenu::Del(Current());
          Display();
      }
   }
   return osContinue;
}

eOSState cRemoteViewMenu::Duplicate()
{
   eOSState status = osContinue;
   cMenuEditKey *editor = CurrentKeyEditor();
   if (!editor)
      return status;

   cKey *key = editor->Key();
   cMenuEditKey *neweditor = new cMenuEditKey(cKey::ToString(key->Key()), editor->Remote(), key, true);

   Add(neweditor, true, editor);
   neweditor->StartLearning();
   Display();


   return osContinue;
}

eOSState cRemoteViewMenu::New()
{
   return osContinue;
}

